@@ -17,7 +17,7 @@ import unittest
 import xml.dom.minidom as dom
 
 from manifestparser import TestManifest
-from marionette import Marionette
+from marionette_driver.marionette import Marionette
 from mixins.b2g import B2GTestResultMixin, get_b2g_pid, get_dm
 from mozhttpd import MozHttpd
 from mozlog.structured.structuredlog import get_default_logger
@@ -25,6 +25,10 @@ from moztest.adapters.unit import StructuredTestRunner, StructuredTestResult
 from moztest.results import TestResultCollection, TestResult, relevant_line
 import mozversion
 
+
+here = os.path.abspath(os.path.dirname(__file__))
+
+
 class MarionetteTest(TestResult):
 
     @property
@@ -238,6 +242,8 @@ class MarionetteTextTestRunner(StructuredTestRunner):
 
 
 class BaseMarionetteOptions(OptionParser):
+    socket_timeout_default = 360.0
+
     def __init__(self, **kwargs):
         OptionParser.__init__(self, **kwargs)
         self.parse_args_handlers = [] # Used by mixins
@@ -376,7 +382,9 @@ class BaseMarionetteOptions(OptionParser):
         self.add_option('--server-root',
                         dest='server_root',
                         action='store',
-                        help='sets the web server\'s root directory to the given path')
+                        help='url to a webserver or path to a document root from which content '
+                        'resources are served (default: {}).'.format(os.path.join(
+                            os.path.dirname(here), 'www')))
         self.add_option('--gecko-log',
                         dest='gecko_log',
                         action='store',
@@ -390,7 +398,25 @@ class BaseMarionetteOptions(OptionParser):
                         action='store',
                         default='Marionette-based Tests',
                         help='Define the name to associate with the logger used')
-
+        self.add_option('--jsdebugger',
+                        dest='jsdebugger',
+                        action='store_true',
+                        default=False,
+                        help='Enable the jsdebugger for marionette javascript.')
+        self.add_option('--pydebugger',
+                        dest='pydebugger',
+                        help='Enable python post-mortem debugger when a test fails.'
+                             ' Pass in the debugger you want to use, eg pdb or ipdb.')
+        self.add_option('--socket-timeout',
+                        dest='socket_timeout',
+                        action='store',
+                        default=self.socket_timeout_default,
+                        help='Set the global timeout for marionette socket operations.')
+        self.add_option('--e10s',
+                        dest='e10s',
+                        action='store_true',
+                        default=False,
+                        help='Enable e10s when running marionette tests.')
 
     def parse_args(self, args=None, values=None):
         options, tests = OptionParser.parse_args(self, args, values)
@@ -440,6 +466,15 @@ class BaseMarionetteOptions(OptionParser):
             if not 1 <= options.this_chunk <= options.total_chunks:
                 self.error('Chunk to run must be between 1 and %s.' % options.total_chunks)
 
+        if options.jsdebugger:
+            options.app_args.append('-jsdebugger')
+            options.socket_timeout = None
+
+        if options.e10s:
+            options.prefs = {
+                'browser.tabs.remote.autostart': True
+            }
+
         for handler in self.verify_usage_handlers:
             handler(options, tests)
 
@@ -459,7 +494,9 @@ class BaseMarionetteTestRunner(object):
                  shuffle=False, shuffle_seed=random.randint(0, sys.maxint),
                  sdcard=None, this_chunk=1, total_chunks=1, sources=None,
                  server_root=None, gecko_log=None, result_callbacks=None,
-                 adb_host=None, adb_port=None, **kwargs):
+                 adb_host=None, adb_port=None, prefs=None,
+                 socket_timeout=BaseMarionetteOptions.socket_timeout_default,
+                 **kwargs):
         self.address = address
         self.emulator = emulator
         self.emulator_binary = emulator_binary
@@ -484,6 +521,7 @@ class BaseMarionetteTestRunner(object):
         self.device_serial = device_serial
         self.symbols_path = symbols_path
         self.timeout = timeout
+        self.socket_timeout = socket_timeout
         self._device = None
         self._capabilities = None
         self._appName = None
@@ -501,6 +539,7 @@ class BaseMarionetteTestRunner(object):
         self.result_callbacks = result_callbacks if result_callbacks is not None else []
         self._adb_host = adb_host
         self._adb_port = adb_port
+        self.prefs = prefs or {}
 
         def gather_debug(test, status):
             rv = {}
@@ -509,10 +548,10 @@ class BaseMarionetteTestRunner(object):
             # In the event we're gathering debug without starting a session, skip marionette commands
             if marionette.session is not None:
                 try:
-                    marionette.set_context(marionette.CONTEXT_CHROME)
-                    rv['screenshot'] = marionette.screenshot()
-                    marionette.set_context(marionette.CONTEXT_CONTENT)
-                    rv['source'] = marionette.page_source
+                    with marionette.using_context(marionette.CONTEXT_CHROME):
+                        rv['screenshot'] = marionette.screenshot()
+                    with marionette.using_context(marionette.CONTEXT_CONTENT):
+                        rv['source'] = marionette.page_source
                 except:
                     logger = get_default_logger()
                     logger.warning('Failed to gather test failure debug.', exc_info=True)
@@ -592,27 +631,32 @@ class BaseMarionetteTestRunner(object):
         self.failures = []
 
     def start_httpd(self, need_external_ip):
-        host = "127.0.0.1"
-        if need_external_ip:
-            host = moznetwork.get_ip()
-        docroot = self.server_root or os.path.join(os.path.dirname(os.path.dirname(__file__)), 'www')
-        if not os.path.isdir(docroot):
-            raise Exception("Server root %s is not a valid path" % docroot)
-        self.httpd = MozHttpd(host=host,
-                              port=0,
-                              docroot=docroot)
-        self.httpd.start()
-        self.marionette.baseurl = 'http://%s:%d/' % (host, self.httpd.httpd.server_port)
-        self.logger.info('running webserver on %s' % self.marionette.baseurl)
-
+        if self.server_root is None or os.path.isdir(self.server_root):
+            host = '127.0.0.1'
+            if need_external_ip:
+                host = moznetwork.get_ip()
+            docroot = self.server_root or os.path.join(os.path.dirname(here), 'www')
+            if not os.path.isdir(docroot):
+                raise Exception('Server root %s is not a valid path' % docroot)
+            self.httpd = MozHttpd(host=host,
+                                  port=0,
+                                  docroot=docroot)
+            self.httpd.start()
+            self.marionette.baseurl = 'http://%s:%d/' % (host, self.httpd.httpd.server_port)
+            self.logger.info('running webserver on %s' % self.marionette.baseurl)
+        else:
+            self.marionette.baseurl = self.server_root
+            self.logger.info('using content from %s' % self.marionette.baseurl)
 
     def _build_kwargs(self):
         kwargs = {
             'device_serial': self.device_serial,
             'symbols_path': self.symbols_path,
             'timeout': self.timeout,
+            'socket_timeout': self.socket_timeout,
             'adb_host': self._adb_host,
             'adb_port': self._adb_port,
+            'prefs': self.prefs,
         }
         if self.bin:
             kwargs.update({
